@@ -4,18 +4,15 @@
 using namespace Rcpp;
 
 // Welford's method
-// http://www.johndcook.com/blog/standard_deviation/
+// https://www.johndcook.com/blog/standard_deviation/
 // [[Rcpp::export]]
 List logSummaryStats(NumericVector x) 
 {
     int N = x.size(), i=0;
-    logRepresentedReal oldM;
-    logRepresentedReal M = logRepresentedReal(x[0], 1);
-    logRepresentedReal S = logRepresentedReal(0, 0);
-    logRepresentedReal thisX;
+    bool seenNA = false;
     
-    NumericVector retM(1);
-    NumericVector retS(1);
+    NumericVector retM(1, NA_REAL);
+    NumericVector retS(1, NA_REAL);
     NumericVector retCumu( N == 0 ? 1 : N, NA_REAL);
     List ret;
     
@@ -24,13 +21,16 @@ List logSummaryStats(NumericVector x)
                  Rcpp::Named("cumLogMean") = retCumu);
                  
     // Ensure that for N=0 and N=1 we return something meaningful 
-    if( N == 0 ){
-      retM(0) = NA_REAL;
-      retS(0) = NA_REAL;
-      retCumu(0) = NA_REAL;
+    if( N == 0 )
       return ret;
-    }
+
+    logRepresentedReal oldM;
+    logRepresentedReal M = logRepresentedReal(x[0], 1);
+    logRepresentedReal S = logRepresentedReal(0, 0);
+    logRepresentedReal thisX;
     
+    seenNA = R_IsNA( x[0] );
+      
     retM(0) = x[0];
     retS(0) = R_NegInf;
     retCumu(0) = x[0];
@@ -40,6 +40,8 @@ List logSummaryStats(NumericVector x)
     
     for( i = 1; i < N; i++ ){
       oldM = M;
+      seenNA = R_IsNA( x[i] );
+      if(seenNA) break;
       thisX = logRepresentedReal( x[i], 1);
       M = oldM + ( thisX - oldM) / double(i + 1);   
       S = S + ( thisX - oldM ) * ( thisX - M );
@@ -47,7 +49,7 @@ List logSummaryStats(NumericVector x)
       retCumu(i) = M.modulo();
     }
   
-    retM(0) = M.modulo();
-    retS(0) = S.modulo() - log(N - 1.0);
+    retM(0) = seenNA ? NA_REAL : M.modulo();
+    retS(0) = seenNA ? NA_REAL : S.modulo() - log(N - 1.0);
     return ret;
 }
